#include "uxn.h"
#include "dat.h"
#include "fns.h"

#include "dev/console.h"
#include "dev/nil.h"
#include "dev/screen.h"
#include "dev/system.h"

Uint8  uxn_ram[64*1024];
Screen uxn_screen;

void
uxn_init(void)
{
	int i;
	Uint8 *rom;

	for(i = 0; i < sizeof(uxn_ram); i++)
		uxn_ram[i] = 0;

	rom = (Uint8 *) ROMADDR;
	for(i = 0; i < 0xf000; i++)
		uxn_ram[PAGE_PROGRAM + i] = rom[i];
}

int
uxn_halt(Uxn *u, Uint8 err, Uint16 addr)
{
	return 0;
}

int
uxn_interrupt(void)
{
	return 0;
}

void
watchdog_init(void)
{
	Uint8 data;

	/* silence the twl4030 watchdog (i hope) */
	data = 0x5e; i2c_write(PHYSI2C1, 0x4a, &data, 1);
	data = 0;    i2c_write(PHYSI2C1, 0x4a, &data, 1);
}

void
main(void)
{
	Uxn u;

	timer_init(PHYSTIMER1);
	timer_init(PHYSTIMER2);
	timer_init(PHYSTIMER3);
	i2c_init(PHYSI2C1);
	i2c_init(PHYSI2C2);

	/* watchdog_init(); NOT WORKING */
	screen_init(&uxn_screen);

	uxn_init();
	uxn_boot(&u, &uxn_ram[0]);

	uxn_port(&u, 0x0, system_dei, system_deo);
	uxn_port(&u, 0x1, nil_dei,    console_deo);
	uxn_port(&u, 0x2, screen_dei, screen_deo);

	uxn_port(&u, 0x3, nil_dei, nil_deo);
	uxn_port(&u, 0x4, nil_dei, nil_deo);
	uxn_port(&u, 0x5, nil_dei, nil_deo);
	uxn_port(&u, 0x6, nil_dei, nil_deo);
	uxn_port(&u, 0x7, nil_dei, nil_deo);
	uxn_port(&u, 0x8, nil_dei, nil_deo);
	uxn_port(&u, 0x9, nil_dei, nil_deo);
	uxn_port(&u, 0xa, nil_dei, nil_deo);
	uxn_port(&u, 0xb, nil_dei, nil_deo);
	uxn_port(&u, 0xc, nil_dei, nil_deo);
	uxn_port(&u, 0xd, nil_dei, nil_deo);
	uxn_port(&u, 0xe, nil_dei, nil_deo);
	uxn_port(&u, 0xf, nil_dei, nil_deo);

	if(!uxn_eval(&u, PAGE_PROGRAM))
		return;

	for(;;) {
		uxn_eval(&u, GETVECTOR(&u.dev[2]));
		if(uxn_screen.fg.changed || uxn_screen.bg.changed)
			screen_redraw(&uxn_screen);

		timer_delay(PHYSTIMER2, 16000);
	}
}

