#include "uxn.h"
#include "dat.h"
#include "fns.h"

#define timer_reg(b, r) *((Uint32*) (b + r))

#define TIMER_PRESCALE 2
#define TIMER_CLOCK    (13000000 / (2 << TIMER_PRESCALE))

static Uint32
timer_read(Uint32 base, Uint8 r)
{
	volatile Uint32* reg = (Uint32*) (base + r);
	return *reg;
}

static void
timer_write(Uint32 base, Uint8 r, Uint32 value)
{
	volatile Uint32* reg = (Uint32*) (base + r);
	*reg = value;
}

void
timer_init(Uint32 base)
{
	/* do a software reset of the timer */
	timer_write(base, 0x10, timer_read(base, 0x10) & 0x2);
	while(!(timer_read(base, 0x14) & 0x1))
		;

	/* load initial value of 0 */
	timer_write(base, 0x2c, 0);
	timer_write(base, 0x24, (TIMER_PRESCALE << 2)
		| (1 << 5)  /* prescaler on */
		| (1 << 1)  /* autoreload on */
		| (1 << 0)); /* start the timer */
}

void
timer_delay(Uint32 base, Uint32 usec)
{
	Sint32 tmo;
	Uint32 now, last;

	tmo = usec * (TIMER_CLOCK / 1000) / 1000;
	last = timer_read(base, 0x28);

	while(tmo > 0) {
		now = timer_read(base, 0x28);
		if(last > now)
			tmo -= 0xffffffff - last + now + 1;
		else
			tmo -= now - last;
		last = now;
	}
}
